# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.cell_attribute_calculator import CellAttributeCalculator


class CellAttributeInformation(CellAttributeCalculator):
    r"""
    CellAttributeInformation - Provide information about a cell
    attribute's basis functions on one cell type.
    
    Superclass: CellAttributeCalculator
    
    Note that some of the methods may be unable to return meaningful
    information about some attribute types; this class has methods that
    may be specific to subclasses of DGCell. If this API does not fit
    your cell type or attribute type; do not register a calculator
    subclass of this type. Instead, it is up to consuming code to ensure
    a non-null calculator is returned.
    
    This calculator assumes interpolation is performed as a weighted sum
    of basis functions evaluated at some parameter value.
    
    \[
       f_j = \sum_i B_i(r) w_{i,j}\]
    
    where $i\in[0, M - 1] $ and $j\in[0,N - 1[ $. + $M $ is
    get_number_of_basis_functions(). + $N $ is
    number_of_components_per_degree_of_freedom().
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkCellAttributeInformation, obj, update, **traits)
    
    def _get_basis_name(self):
        return self._vtk_obj.GetBasisName()
    basis_name = traits.Property(_get_basis_name, desc=\
        r"""
        Return a name for a basis function specific to both the
        cell-metadata and the cell-attribute type.
        
        This is used by some render responders and interpolation
        calculators to choose a function to invoke.
        """
    )

    def _get_basis_order(self):
        return self._vtk_obj.GetBasisOrder()
    basis_order = traits.Property(_get_basis_order, desc=\
        r"""
        Return the polynomial order of the interpolant.
        
        Subclasses must override this method to perform evaluation.
        """
    )

    def _get_basis_value_size(self):
        return self._vtk_obj.GetBasisValueSize()
    basis_value_size = traits.Property(_get_basis_value_size, desc=\
        r"""
        Return the number of components generated for each basis
        function.
        
        For HGRAD function spaces, this is 1. For HCURL or HDIV function
        spaces, this is equal to the parametric dimension of the cell
        type in question.
        """
    )

    def _get_degree_of_freedom_size(self):
        return self._vtk_obj.GetDegreeOfFreedomSize()
    degree_of_freedom_size = traits.Property(_get_degree_of_freedom_size, desc=\
        r"""
        Return the number of times each basis function should be applied
        to obtain an attribute value.
        
        For example, each HGRAD basis function for a hexahedron returns a
        single value per degree of freedom. However, when the output
        attribute has 3 components (say, x, y, and z point coordinates),
        then the weighted sum of the basis functions must be applied 3
        times (once for each point coordinate). This is how vector- or
        tensor-valued attributes may be composed from scalar basis
        functions.
        """
    )

    def _get_number_of_basis_functions(self):
        return self._vtk_obj.GetNumberOfBasisFunctions()
    number_of_basis_functions = traits.Property(_get_number_of_basis_functions, desc=\
        r"""
        Return the number of basis functions for the cell type in
        question.
        """
    )

    def _get_shared_degrees_of_freedom(self):
        return self._vtk_obj.GetSharedDegreesOfFreedom()
    shared_degrees_of_freedom = traits.Property(_get_shared_degrees_of_freedom, desc=\
        r"""
        Return whether degrees of freedom are shared by multiple cells or
        not.
        
        This roughly translates to whether the attribute is allowed to
        express discontinuities at cell boundaries; when degrees of
        freedom are shared, continuity is presumed. Otherwise, the
        interpolation functions may not produce identical values where
        cell boundaries overlap.
        """
    )

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(CellAttributeInformation, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit CellAttributeInformation properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit CellAttributeInformation properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit CellAttributeInformation properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

