# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.cell_grid_query import CellGridQuery


class CellGridEvaluator(CellGridQuery):
    r"""
    CellGridEvaluator - Evaluate a field (vtkcell_attribute) at some
    points inside cells.
    
    Superclass: CellGridQuery
    
    This class is a cell-grid query whose purpose is to determine the
    value a CellAttribute takes on at one or more points inside the
    domain of a CellGrid.
    
    This class performs its work in two phases: + Classification. Input
    points are classified by the type and index
      of cell in the grid in which they lie. + Attribute/field
    interpolation. Each cell is asked to interpolate
      the value of a cell-attribute at each point classified to it.
    
    As an example, consider a cell-grid holding 10 triangles and 20 quads
    and a query that is provided 5 points. The first phase will identify
    which of the 5 points are insides triangles, which lie in
    quadrilaterals, and which lie in neither. Say that 2 lie inside
    triangles, 2 inside quadrilaterals, and 1 is outside the domain.
    Furthermore, the first phase will identify which particular triangles
    or quadrilaterals contain the input points. The two points which lie
    in triangles will report a number in [0,10[ while the two points
    which lie in quadrilaterals will report a number in [0, 20[. Finally,
    for cells which have a reference element, the parametric coordinates
    of each input point are computed.
    
    The second phase additionally interpolates a cell-attribute (let's
    say "Velocity" in our example) at each input point.
    
    You may configure the query to skip either phase (classification or
    interpolation). If you skip classification, you must provide the the
    classification information for the input points. The method you call
    (classify_points, interpolate_points, or interpolate_cell_parameters)
    determines which phase(s) are applied during evaluation.
    
    When running in interpolate_points mode (both classification and
    interpolation phases are performed), the output from our example is
    reported like so:
    
    + `get_classifier_cell_types()` – returns an array with a cell-type
    hash
      for each type of cell containing an input point. The hash value can
      be used to construct a StringToken.
      Our example would return an array with 3 values which might be
      ordered: "DGTri"_hash, "DGQuad"_hash, and 0 (an "invalid"
    hash). + `get_classifier_cell_offsets()` – returns an array with the
    same number
      of values as the call above. Each value specifies the start of
      reporting for points contained in the corresponding cell type.
      Our example would return [0, 2, 4] to match the values above. +
    `get_classifier_point_i_ds()` – returns an array whose length matches
      the number of input points. Each value is the index of an input
      point. Input points do not have their order preserved so that
      all the points contained in a single cell can be reported together.
      Our example might return [4, 2, 1, 0, 3]. This will always be a
      permutation of the counting integers and, for our example, always
      hold integers in [0, 5[. + `get_classifier_cell_indices()` – returns
    an array whose length matches
      the number of input points. Each value is the index into cells
      of the corresponding type, indicating which cell contains
      the input point.
      For our example, the first two numbers will be in [0, 10[, the
    second
      two will be in [0, 20[, and the last will be -1. (This is because
      we have two points inside 10 triangles, two points inside 20 quads,
      and one un-classifiable input point.) +
    `get_classifier_point_parameters()` – returns an array whose length
      matches the number of input points. Each value is a 3-tuple of
      reference-cell coordinates (or indeterminate if the cell type does
      not provide a reference cell). + `get_interpolated_values()` –
    returns an array whose number of tuples
      matches the number of input points and whose number of components
      matches the number of components of the requested cell-attribute.
      For our example, an array with 5 tuples of 3 components each would
    be
      returned; it would be named "Velocity" (matching the
    cell-attribute's
      name).
    
    Note that because you can pass in the arrays above (except the
    interpolated values) to short-circuit classification, it is possible
    to evaluate multiple cell-attributes without duplicating the
    classification work.
    
    In interpolate_cell_parameters mode, calling the methods above which
    begin with `get_classifier…` will simply return the input arrays you
    passed to configure the query.
    
    ## Warnings
    
    The output arrays above generally match the number of input points,
    but will sometimes exceed the number of input points. This will occur
    when multiple cells contain an input point (either on a shared
    boundary or because the cells overlap).
    
    Note that the output should never have fewer points than the input as
    even points outside the cells will be classified as such.
    
    Currently, this class is limited to evaluating numeric attributes;
    string or variant arrays are not supported.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkCellGridEvaluator, obj, update, **traits)
    
    def _get_cell_attribute(self):
        return wrap_vtk(self._vtk_obj.GetCellAttribute())
    def _set_cell_attribute(self, arg):
        old_val = self._get_cell_attribute()
        self._wrap_call(self._vtk_obj.SetCellAttribute,
                        deref_vtk(arg))
        self.trait_property_changed('cell_attribute', old_val, arg)
    cell_attribute = traits.Property(_get_cell_attribute, _set_cell_attribute, desc=\
        r"""
        
        """
    )

    def _get_classifier_cell_indices(self):
        return wrap_vtk(self._vtk_obj.GetClassifierCellIndices())
    classifier_cell_indices = traits.Property(_get_classifier_cell_indices, desc=\
        r"""
        Get the array holding indices into cells of a given type that
        contain input points.
        """
    )

    def _get_classifier_cell_offsets(self):
        return wrap_vtk(self._vtk_obj.GetClassifierCellOffsets())
    classifier_cell_offsets = traits.Property(_get_classifier_cell_offsets, desc=\
        r"""
        Get the offset into the result arrays where each cell-type's
        points reside.
        """
    )

    def _get_classifier_cell_types(self):
        return wrap_vtk(self._vtk_obj.GetClassifierCellTypes())
    classifier_cell_types = traits.Property(_get_classifier_cell_types, desc=\
        r"""
        Get the discovered types of cells containing input points.
        """
    )

    def _get_classifier_point_i_ds(self):
        return wrap_vtk(self._vtk_obj.GetClassifierPointIDs())
    classifier_point_i_ds = traits.Property(_get_classifier_point_i_ds, desc=\
        r"""
        Get the array holding indices into the input_points array of each
        output.
        """
    )

    def _get_classifier_point_parameters(self):
        return wrap_vtk(self._vtk_obj.GetClassifierPointParameters())
    classifier_point_parameters = traits.Property(_get_classifier_point_parameters, desc=\
        r"""
        Get the array holding (r, s, t)-coordinates where each input
        point is located.
        """
    )

    def _get_input_points(self):
        return wrap_vtk(self._vtk_obj.GetInputPoints())
    input_points = traits.Property(_get_input_points, desc=\
        r"""
        Get the input points to be classified and possibly interpolated.
        These points are in world coordinates (not in reference cell
        coordinates).
        """
    )

    def _get_interpolated_values(self):
        return wrap_vtk(self._vtk_obj.GetInterpolatedValues())
    interpolated_values = traits.Property(_get_interpolated_values, desc=\
        r"""
        Get the array holding cell-attribute values interpolated to each
        input point.
        """
    )

    def _get_locator(self):
        return wrap_vtk(self._vtk_obj.GetLocator())
    locator = traits.Property(_get_locator, desc=\
        r"""
        
        """
    )

    def get_phases_to_perform(self, *args):
        """
        get_phases_to_perform(self) -> CellGridEvaluator.Phases
        C++: virtual CellGridEvaluator::Phases get_phases_to_perform()
        Return what work the query has been configured to do.
        """
        ret = self._wrap_call(self._vtk_obj.GetPhasesToPerform, *args)
        return ret

    def classify_points(self, *args):
        """
        classify_points(self, points:DataArray) -> None
        C++: void classify_points(DataArray *points)
        Configure the query to run the classifier but not the
        interpolator.
        """
        my_args = deref_array(args, [['vtkDataArray']])
        ret = self._wrap_call(self._vtk_obj.ClassifyPoints, *my_args)
        return ret

    def interpolate_cell_parameters(self, *args):
        """
        interpolate_cell_parameters(self, cellTypes:TypeUInt32Array,
            cellOffsets:TypeUInt64Array,
            cellIndices:TypeUInt64Array, pointParameters:DataArray)
            -> None
        C++: void interpolate_cell_parameters(TypeUInt32Array *cellTypes,
             TypeUInt64Array *cellOffsets,
            TypeUInt64Array *cellIndices,
            DataArray *pointParameters)
        Configure the query to run only the interpolator.
        
        Because interpolation usually requires classification information
        and parametric coordinates for each classified point, you must
        provide arrays holding this information.
        """
        my_args = deref_array(args, [('vtkTypeUInt32Array', 'vtkTypeUInt64Array', 'vtkTypeUInt64Array', 'vtkDataArray')])
        ret = self._wrap_call(self._vtk_obj.InterpolateCellParameters, *my_args)
        return ret

    def interpolate_points(self, *args):
        """
        interpolate_points(self, points:DataArray) -> None
        C++: void interpolate_points(DataArray *points)
        Configure the query to run the classifier followed by the
        interpolator.
        """
        my_args = deref_array(args, [['vtkDataArray']])
        ret = self._wrap_call(self._vtk_obj.InterpolatePoints, *my_args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(CellGridEvaluator, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit CellGridEvaluator properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit CellGridEvaluator properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit CellGridEvaluator properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

