# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.poly_data_algorithm import PolyDataAlgorithm


class DecimatePolylineFilter(PolyDataAlgorithm):
    r"""
    DecimatePolylineFilter - reduce the number of lines in a polyline
    
    Superclass: PolyDataAlgorithm
    
    DecimatePolylineFilter is a filter to reduce the number of lines
    in a polyline. The algorithm functions by evaluating an error metric
    for each vertex (i.e., the distance of the vertex to a line defined
    from the two vertices on either side of the vertex). This error
    metric is computed through strategies, there are multiple
    implementations available and the default one is
    `vtkdecimate_polyline_distance_strategy`. Then, these vertices are
    placed into a priority queue, and those with smaller errors are
    deleted first. The decimation continues until the target reduction is
    reached. While the filter will not delete end points, it will
    decimate closed loops down to a single line, thereby changing
    topology.
    
    Note that a maximum error value (which expression depends on the
    strategy used) can also be specified. This may limit the amount of
    decimation so the target reduction may not be met. When using the
    `vtkdecimate_polyline_distance_strategy`, setting the maximum error
    value to a very small number, will eliminate colinear points.
    
    @warning
    This algorithm is a very simple implementation that overlooks some
    potential complexities. For example, if a vertex is multiply
    connected, meaning that it is used by multiple distinct polylines,
    then the extra topological constraints are ignored. This can produce
    less than optimal results.
    
    @sa
    Decimate DecimateProp QuadricClustering QuadricDecimation
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkDecimatePolylineFilter, obj, update, **traits)
    
    def _get_decimation_strategy(self):
        return wrap_vtk(self._vtk_obj.GetDecimationStrategy())
    def _set_decimation_strategy(self, arg):
        old_val = self._get_decimation_strategy()
        self._wrap_call(self._vtk_obj.SetDecimationStrategy,
                        deref_vtk(arg))
        self.trait_property_changed('decimation_strategy', old_val, arg)
    decimation_strategy = traits.Property(_get_decimation_strategy, _set_decimation_strategy, desc=\
        r"""
        
        """
    )

    maximum_error = traits.Trait(1e+299, traits.Range(0.0, 1e+299, enter_set=True, auto_set=False), desc=\
        r"""
        Set the largest decimation error that is allowed during the
        decimation process. This may limit the maximum reduction that may
        be achieved. The maximum error is dependent on the decimation
        strategy used, by default it is specified as a fraction of the
        maximum length of the input data bounding box. Defaults to
        VTK_DOUBLE_MAX.
        """
    )

    def _maximum_error_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetMaximumError,
                        self.maximum_error)

    output_points_precision = traits.Int(2, enter_set=True, auto_set=False, desc=\
        r"""
        Set/get the desired precision for the output types. See the
        documentation for the Algorithm::DesiredOutputPrecision enum
        for an explanation of the available precision settings. Defaults
        to DEFAULT_PRECISION.
        """
    )

    def _output_points_precision_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOutputPointsPrecision,
                        self.output_points_precision)

    target_reduction = traits.Trait(0.9, traits.Range(0.0, 1.0, enter_set=True, auto_set=False), desc=\
        r"""
        Specify the desired reduction in the total number of polygons
        (e.g., if target_reduction is set to 0.9, this filter will try to
        reduce the data set to 10% of its original size). Defaults to 0.9
        .
        """
    )

    def _target_reduction_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetTargetReduction,
                        self.target_reduction)

    def _get_input(self):
        try:
            return wrap_vtk(self._vtk_obj.GetInput(0))
        except TypeError:
            return wrap_vtk(self._vtk_obj.GetInput())
    input = traits.Property(_get_input,
                            desc="The first input of this object, i.e. the result of `get_input(0)`.")
    
    def get_input(self, *args):
        """
        get_input(self) -> DataObject
        C++: DataObject *get_input()
        get_input(self, port:int) -> DataObject
        C++: DataObject *get_input(int port)"""
        ret = self._wrap_call(self._vtk_obj.GetInput, *args)
        return wrap_vtk(ret)

    _updateable_traits_ = \
    (('abort_execute', 'GetAbortExecute'), ('release_data_flag',
    'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'),
    ('maximum_error', 'GetMaximumError'), ('output_points_precision',
    'GetOutputPointsPrecision'), ('target_reduction',
    'GetTargetReduction'), ('abort_output', 'GetAbortOutput'),
    ('progress_text', 'GetProgressText'), ('object_name',
    'GetObjectName'), ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'global_warning_display',
    'release_data_flag', 'abort_output', 'maximum_error', 'object_name',
    'output_points_precision', 'progress_text', 'target_reduction'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(DecimatePolylineFilter, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit DecimatePolylineFilter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['abort_output', 'maximum_error', 'object_name',
            'output_points_precision', 'target_reduction']),
            title='Edit DecimatePolylineFilter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit DecimatePolylineFilter properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

