# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.cell_metadata import CellMetadata


class DGCell(CellMetadata):
    r"""
    DGCell - Base class for a discontinuous Galerkin cells of all
    shapes.
    
    Superclass: CellMetadata
    
    This class exists to offer each shape's parameterization via a
    uniform API.
    
    All DG cells have shapes that can be described by corner points in a
    reference (parametric) coordinate system. Sides (boundaries) of the
    element of any dimension can be fetched as offsets into the list of
    corners. You can also obtain a list of the coordinates in parametric
    space of all the corner points.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkDGCell, obj, update, **traits)
    
    def get_cell_source_connectivity(self, *args):
        """
        get_cell_source_connectivity(self, sideType:int=-1) -> DataArray
        C++: DataArray *get_cell_source_connectivity(int sideType=-1)
        Python-accessible methods to fetch cell source information.
        
        In C++, you should simply call get_cell_source( sideType) and
        access the ivar.
        """
        ret = self._wrap_call(self._vtk_obj.GetCellSourceConnectivity, *args)
        return wrap_vtk(ret)

    def get_cell_source_is_blanked(self, *args):
        """
        get_cell_source_is_blanked(self, sideType:int=-1) -> bool
        C++: bool get_cell_source_is_blanked(int sideType=-1)"""
        ret = self._wrap_call(self._vtk_obj.GetCellSourceIsBlanked, *args)
        return ret

    def get_cell_source_nodal_ghost_marks(self, *args):
        """
        get_cell_source_nodal_ghost_marks(self, sideType:int=-1)
            -> DataArray
        C++: DataArray *get_cell_source_nodal_ghost_marks(int sideType=-1)"""
        ret = self._wrap_call(self._vtk_obj.GetCellSourceNodalGhostMarks, *args)
        return wrap_vtk(ret)

    def get_cell_source_offset(self, *args):
        """
        get_cell_source_offset(self, sideType:int=-1) -> int
        C++: IdType get_cell_source_offset(int sideType=-1)"""
        ret = self._wrap_call(self._vtk_obj.GetCellSourceOffset, *args)
        return ret

    def get_cell_source_selection_type(self, *args):
        """
        get_cell_source_selection_type(self, sideType:int=-1) -> int
        C++: int get_cell_source_selection_type(int sideType=-1)"""
        ret = self._wrap_call(self._vtk_obj.GetCellSourceSelectionType, *args)
        return ret

    def get_cell_source_shape(self, *args):
        """
        get_cell_source_shape(self, sideType:int=-1) -> Shape
        C++: Shape get_cell_source_shape(int sideType=-1)"""
        ret = self._wrap_call(self._vtk_obj.GetCellSourceShape, *args)
        return ret

    def get_cell_source_side_type(self, *args):
        """
        get_cell_source_side_type(self, sideType:int=-1) -> int
        C++: int get_cell_source_side_type(int sideType=-1)"""
        ret = self._wrap_call(self._vtk_obj.GetCellSourceSideType, *args)
        return ret

    def _get_dimension(self):
        return self._vtk_obj.GetDimension()
    dimension = traits.Property(_get_dimension, desc=\
        r"""
        Return the parametric dimension of this cell type (0, 1, 2, or
        3).
        """
    )

    def _get_number_of_cell_sources(self):
        return self._vtk_obj.GetNumberOfCellSources()
    number_of_cell_sources = traits.Property(_get_number_of_cell_sources, desc=\
        r"""
        Python-accessible method to identify number of cell sources.
        """
    )

    def _get_number_of_corners(self):
        return self._vtk_obj.GetNumberOfCorners()
    number_of_corners = traits.Property(_get_number_of_corners, desc=\
        r"""
        Return the number of corner points for this cell type.
        """
    )

    def _get_number_of_side_types(self):
        return self._vtk_obj.GetNumberOfSideTypes()
    number_of_side_types = traits.Property(_get_number_of_side_types, desc=\
        r"""
        Return the number of different side shapes of this cell type.
        
        Example: a wedge has 4 side shapes: Quadrilateral, Triangle,
        Edge, and Vertex.
        """
    )

    def get_number_of_sides_of_dimension(self, *args):
        """
        get_number_of_sides_of_dimension(self, dimension:int) -> int
        C++: virtual int get_number_of_sides_of_dimension(int dimension)
        Return the number of boundaries this type of cell has of a given
        dimension.
        
        DG cells can be thought of as CW-complex cells; they are tuples
        of corner points which represent an open point set plus its
        closure decomposed into a union of open sets of lower dimension.
        For example, a hexahedron is an 8-tuple of corner points
        representing an underlying space shaped as an open, rectangular
        prism plus six 4-tuples representing its quadrilateral faces,
        plus twelve 2-tuples representing its edges, plus 8 1-tuples
        representing its corners. Thus, a hexahedron has 6 + 12 + 8 = 26
        sides (plus its interior).
        
        Passing a dimension of -1 will return 1 side; DG cells use a
        -1-dimensional side to indicate an entire cell should be treated
        as a side. This is useful for subsetting cells without re-writing
        the arrays holding connectivity and field values; it also allows
        a DGCell instance to hold both sides of input cells mixed with
        a subset of the input cells. For example, given 3 triangles as
        input, generating an output with 1 input triangle unchanged and
        edges or vertices from the other two triangles requires per-cell
        blanking or subsetting of cells.
        """
        ret = self._wrap_call(self._vtk_obj.GetNumberOfSidesOfDimension, *args)
        return ret

    def get_parametric_center_of_side(self, *args):
        """
        get_parametric_center_of_side(self, sideId:int) -> Vector3d
        C++: virtual Vector3d get_parametric_center_of_side(int sideId)
        Return the parametric center of a cell or its side.
        
        Pass -1 for side if you want the cell's center. Otherwise, pass
        the side ID.
        
        This method simply averages corner-point coordinates. It is not
        fast, since it averages values each time it is called. If you
        need to re-use this information, you are responsible for caching
        it locally.
        """
        ret = self._wrap_call(self._vtk_obj.GetParametricCenterOfSide, *args)
        return wrap_vtk(ret)

    def _get_reference_points(self):
        return wrap_vtk(self._vtk_obj.GetReferencePoints())
    reference_points = traits.Property(_get_reference_points, desc=\
        r"""
        Return a singleton array initialized with the reference-cell's
        corner point coordinates.
        
        When implementing a subclass, call fill_reference_points() in your
        override. This should always return the same TypeFloat32Array
        each time so that the array is only uploaded to the GPU once.
        """
    )

    def _get_shape(self):
        return self._vtk_obj.GetShape()
    shape = traits.Property(_get_shape, desc=\
        r"""
        Return the topological shape of this cell or side type.
        """
    )

    def get_shape_corner_count(self, *args):
        """
        get_shape_corner_count(shape:Shape) -> int
        C++: static int get_shape_corner_count(Shape shape)"""
        ret = self._wrap_call(self._vtk_obj.GetShapeCornerCount, *args)
        return ret

    def get_shape_dimension(self, *args):
        """
        get_shape_dimension(shape:Shape) -> int
        C++: static int get_shape_dimension(Shape shape)"""
        ret = self._wrap_call(self._vtk_obj.GetShapeDimension, *args)
        return ret

    def get_shape_enum(self, *args):
        """
        get_shape_enum(shapeName:StringToken) -> Shape
        C++: static Shape get_shape_enum(StringToken shapeName)
        Given a string description of a cell shape, return the DG
        equivalent enum.
        
        Note that this also converts IOSS shape names to DG enums, so
        there are additional cases to handle spheres as points, springs
        as lines, etc.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.GetShapeEnum, *my_args)
        return ret

    def get_shape_name(self, *args):
        """
        get_shape_name(shape:Shape) -> StringToken
        C++: static StringToken get_shape_name(Shape shape)"""
        ret = self._wrap_call(self._vtk_obj.GetShapeName, *args)
        return wrap_vtk(ret)

    def _get_side_connectivity(self):
        return wrap_vtk(self._vtk_obj.GetSideConnectivity())
    side_connectivity = traits.Property(_get_side_connectivity, desc=\
        r"""
        Return the connectivity of the given side.
        
        The side connectivity is a vector holding the indexes of
        corner-points of the side into the connectivity vector of this
        cell.
        
        Passing a side of -1 should return the connectivity of the cell's
        interior (a vector of the counting numbers from 0 to
        this->get_number_of_corners()). This feature is used when rendering
        cells of dimension 2 or lower.
        """
    )

    def get_side_connectivity(self, *args):
        """
        get_side_connectivity(self, side:int) -> (int, ...)
        C++: virtual const std::vector<IdType> &get_side_connectivity(
            int side)
        get_side_connectivity(self) -> TypeInt32Array
        C++: virtual TypeInt32Array *get_side_connectivity()
        Return the connectivity of the given side.
        
        The side connectivity is a vector holding the indexes of
        corner-points of the side into the connectivity vector of this
        cell.
        
        Passing a side of -1 should return the connectivity of the cell's
        interior (a vector of the counting numbers from 0 to
        this->get_number_of_corners()). This feature is used when rendering
        cells of dimension 2 or lower.
        """
        ret = self._wrap_call(self._vtk_obj.GetSideConnectivity, *args)
        return wrap_vtk(ret)

    def _get_side_offsets_and_shapes(self):
        return wrap_vtk(self._vtk_obj.GetSideOffsetsAndShapes())
    side_offsets_and_shapes = traits.Property(_get_side_offsets_and_shapes, desc=\
        r"""
        Return a singleton array initialized with 2-tuples of (offset,
        shape) values.
        
        When implementing a subclass, call fill_side_offsets_and_shapes() in
        your override. This should always return the same
        TypeInt32Array each time so that the array is only uploaded to
        the GPU once.
        """
    )

    def get_side_range_for_side_dimension(self, *args):
        """
        get_side_range_for_side_dimension(self, sideDimension:int) -> (int,
            int)
        C++: int *get_side_range_for_side_dimension(int sideDimension)
        A python-wrapped version of get_side_range_for_dimension.
        """
        ret = self._wrap_call(self._vtk_obj.GetSideRangeForSideDimension, *args)
        return ret

    def get_side_range_for_side_type(self, *args):
        """
        get_side_range_for_side_type(self, sideType:int) -> (int, int)
        C++: int *get_side_range_for_side_type(int sideType)
        A python-wrapped version of get_side_range_for_type.
        """
        ret = self._wrap_call(self._vtk_obj.GetSideRangeForSideType, *args)
        return ret

    def get_side_shape(self, *args):
        """
        get_side_shape(self, side:int) -> Shape
        C++: virtual Shape get_side_shape(int side)
        For a given side, return its cell shape.
        
        The sides of a DGCell are always ordered from highest
        dimension to lowest. For example, a hexahedron's quadrilateral
        sides are numbered 0–5, its line sides are numbered 6–17, and
        its corner point sides are numbered 18–25. Sometimes, the
        interior of the element is considered a side labeled -1.
        """
        ret = self._wrap_call(self._vtk_obj.GetSideShape, *args)
        return ret

    def get_side_type_for_shape(self, *args):
        """
        get_side_type_for_shape(self, s:Shape) -> int
        C++: virtual int get_side_type_for_shape(Shape s)
        Return the side type index for the given shape (or -1).
        
        This method simply loops over the cell's side types until it
        finds one of the proper shape.
        """
        ret = self._wrap_call(self._vtk_obj.GetSideTypeForShape, *args)
        return ret

    def get_sides_of_side(self, *args):
        """
        get_sides_of_side(self, side:int) -> (int, ...)
        C++: virtual const std::vector<IdType> &get_sides_of_side(
            int side)
        Return a vector of side IDs given an input side ID.
        
        Passing a side of -1 will return the sides of the element itself.
        The returned values **are not** corner-point IDs; they **are**
        side IDs. You can call get_side_connectivity on each entry of the
        returned vector to obtain corner-point IDs.
        """
        ret = self._wrap_call(self._vtk_obj.GetSidesOfSide, *args)
        return ret

    def fill_reference_points(self, *args):
        """
        fill_reference_points(self, arr:TypeFloat32Array) -> None
        C++: void fill_reference_points(TypeFloat32Array *arr)
        Fill the passed array with the parametric coordinates of all the
        element's corners.
        """
        my_args = deref_array(args, [['vtkTypeFloat32Array']])
        ret = self._wrap_call(self._vtk_obj.FillReferencePoints, *my_args)
        return ret

    def fill_side_connectivity(self, *args):
        """
        fill_side_connectivity(self, arr:TypeInt32Array) -> None
        C++: void fill_side_connectivity(TypeInt32Array *arr)
        Fill the passed array with the connectivity (point IDs) of all
        the element's sides.
        """
        my_args = deref_array(args, [['vtkTypeInt32Array']])
        ret = self._wrap_call(self._vtk_obj.FillSideConnectivity, *my_args)
        return ret

    def fill_side_offsets_and_shapes(self, *args):
        """
        fill_side_offsets_and_shapes(self, arr:TypeInt32Array) -> None
        C++: void fill_side_offsets_and_shapes(TypeInt32Array *arr)
        Fill the passed array with tuples of (1) offsets into the
        side-connectivity and (2) shapes for each type of side. Note that
        the final tuple contains the total size of the offset array and a
        shape corresponding to the element itself.
        
        Each element's vertex side-connectivity (the penultimate offset)
        can also be used as the connectivity for the element's
        connectivity.
        
        Simple example: a DGTri has 3 tuples: + (0, Shape::Edge), +
        (3, Shape::Vertex), + (6, Shape::Triangle).
        
        Complex example: a DGWedge has 5 tuples: + (0,
        Shape::Quadrilateral), + (3, Shape::Triangle), + (5,
        Shape::Edge), + (14, Shape::Vertex), + (20, Shape::Wedge).
        
        Note that the wedge has multiple 2-d sides (both quadilaterals
        and triangles).
        """
        my_args = deref_array(args, [['vtkTypeInt32Array']])
        ret = self._wrap_call(self._vtk_obj.FillSideOffsetsAndShapes, *my_args)
        return ret

    def is_inside(self, *args):
        """
        is_inside(self, rst:Vector3d, tolerance:float=1e-6) -> bool
        C++: virtual bool is_inside(const Vector3d &rst,
            double tolerance=1e-6)
        Return true if the parametric coordinates ( rst) lie inside the
        reference cell or its closure and false otherwise.
        
        The tolerance specifies a margin that should be included as part
        of the reference cell's interior to account for numerical
        imprecision.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.IsInside, *my_args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(DGCell, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit DGCell properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit DGCell properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit DGCell properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

