# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.partitioned_data_set_collection_algorithm import PartitionedDataSetCollectionAlgorithm


class EnSightSOSGoldReader(PartitionedDataSetCollectionAlgorithm):
    r"""
    EnSightSOSGoldReader - reader for en_sight SOS gold files
    
    Superclass: PartitionedDataSetCollectionAlgorithm
    
    Reads en_sight SOS files using EnSightGoldCombinedReader. When
    running in parallel, the decomposition of the data across ranks
    differs in this reader from the parallel en_sight readers available in
    para_view source (see
    https://kitware.github.io/paraview-docs/latest/cxx/classPVen_sight_ma
    sterserver_reader2.html,
    https://kitware.github.io/paraview-docs/latest/cxx/classPen_sight_gol
    dbinary_reader.html, and
    https://kitware.github.io/paraview-docs/latest/cxx/classPen_sight_gol
    dReader.html). This reader makes the assumption that most users will
    already have a good decomposition in the way their files are written
    out from their solvers, so we will honor that.
    
    The format allows for partitions of parts to be contained in
    different casefiles making up the SOS file. The old reader treated
    these partitions of the same part across casefiles as different parts
    that were not related to each other, so then a part that is split
    across casefiles would end up with a very weird decomposition.
    
    The initial strategy for this reader assigns whole casefiles to the
    available processes as evenly as possible. This means that using the
    same number of processes as number of servers listed in the SOS file
    is the most efficient - each rank will read one casefile. If there
    are more processes than casefiles, some ranks will do no work, while
    if there are fewer processes than casefiles, some rank(s) will read
    more than one casefile. In the future, we will add a strategy that
    will consider the partition of a part in the casefile as an atomic
    unit, and those partitions could be more evenly distributed across
    ranks.
    
    The output of the reader is a PartitionedDataSetCollection. When a
    process reads multiple casefiles, it will combine the output
    PartitionedDataSetCollection from each of the
    EnSightGoldCombinedReader instances to output a single
    PartitionedDataSetCollection where the portions of parts read by
    different ranks are partitions of their respective
    PartitionedDataSet. For instance if 'Part 1' is empty in
    casefile1, and has its data split between casefile2 and casefile3,
    the resulting PartitionedDataSetCollection will contain only one
    PartitionedDataSet containing 'Part 1' with two partitions.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkEnSightSOSGoldReader, obj, update, **traits)
    
    case_file_name = tvtk_base.vtk_file_name("", desc=\
        r"""
        
        """
    )

    def _case_file_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetCaseFileName,
                        self.case_file_name)

    def _get_controller(self):
        return wrap_vtk(self._vtk_obj.GetController())
    def _set_controller(self, arg):
        old_val = self._get_controller()
        self._wrap_call(self._vtk_obj.SetController,
                        deref_vtk(arg))
        self.trait_property_changed('controller', old_val, arg)
    controller = traits.Property(_get_controller, _set_controller, desc=\
        r"""
        Get/Set the controller
        """
    )

    def _get_cell_array_selection(self):
        return wrap_vtk(self._vtk_obj.GetCellArraySelection())
    cell_array_selection = traits.Property(_get_cell_array_selection, desc=\
        r"""
        Cell array selection, to determine which cell arrays are loaded.
        """
    )

    def _get_field_array_selection(self):
        return wrap_vtk(self._vtk_obj.GetFieldArraySelection())
    field_array_selection = traits.Property(_get_field_array_selection, desc=\
        r"""
        Field data array selection, to determine which arrays are loaded.
        """
    )

    def _get_part_selection(self):
        return wrap_vtk(self._vtk_obj.GetPartSelection())
    part_selection = traits.Property(_get_part_selection, desc=\
        r"""
        Part selection, to determine which blocks/parts are loaded.
        """
    )

    def _get_point_array_selection(self):
        return wrap_vtk(self._vtk_obj.GetPointArraySelection())
    point_array_selection = traits.Property(_get_point_array_selection, desc=\
        r"""
        Point array selection, to determine which point arrays are
        loaded.
        """
    )

    def can_read_file(self, *args):
        """
        can_read_file(self, fname:str) -> int
        C++: int can_read_file(const char *fname)"""
        ret = self._wrap_call(self._vtk_obj.CanReadFile, *args)
        return ret

    _updateable_traits_ = \
    (('abort_execute', 'GetAbortExecute'), ('release_data_flag',
    'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'),
    ('case_file_name', 'GetCaseFileName'), ('abort_output',
    'GetAbortOutput'), ('progress_text', 'GetProgressText'),
    ('object_name', 'GetObjectName'), ('reference_count',
    'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'global_warning_display',
    'release_data_flag', 'abort_output', 'case_file_name', 'object_name',
    'progress_text'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(EnSightSOSGoldReader, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit EnSightSOSGoldReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['abort_output', 'case_file_name', 'object_name']),
            title='Edit EnSightSOSGoldReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit EnSightSOSGoldReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

