# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.partitioned_data_set_collection_algorithm import PartitionedDataSetCollectionAlgorithm


class ERFReader(PartitionedDataSetCollectionAlgorithm):
    r"""
    ERFReader - Read ERF-HDF5 files.
    
    Superclass: PartitionedDataSetCollectionAlgorithm
    
    Based on HDF5, ERF format will contain multiple stages and a special
    group named 'post' which indicates the default stage.
    
    A stage is a group with multiple subgroups which could be mandatory
    or not:
    - erfheader   (mandatory): defines the version of the format /
      simulation.
    - constant    (mandatory): defines all data which is not varying.
    - singlestate (opt)      : defines multiple states with varying data.
    - multistate  (opt)      : defines data which can depend on multiple
      varying parameters like time series and load cases.
    
    Here is an example of a valid ERF HDF5 file hierarchy: /
     CSMEXPL/
      constant/
       connectivities/
        SHELL/
         erfblock/
          cell indices
          connecitvity indices
          offsets
       entityresults/
        NODE/
         erfblock/
          indices
          points
          offsets
       systems/
         erfblock/
      erfheader/
      multistate/                (currently not supported yet)
      singlestate/
       state00001/
        ...
       state00004/
        ...
     post/
    
    Each subgroup of stage, like 'constant', will also contains multiple
    groups (e.g. 'connectivities') and so on. The deeper subgroup, for
    each possible path, named `erfblock` has several handy attributes to
    generate the data in VTK like: dimension, cell type, associated
    groups, ... Also 'erfblock' contains several datasets (which is a
    file in HDF5) which will be the raw data like points, indices, ...
    
    This reader will output a PartitionedDataSetCollection as it
    should be read in distributed fashion later and it represents a
    collection of dataSet which can vary depending on the time
    (singlestate, multistate) or not (constant).
    
    @warning Multistate isn't supported for now.
    
    ote for more details, the spec about ERF HDF5 implemented can be
    found here:
    https://myesi.esi-group.com/ERF-HDF5/doc/ERF_HDF5_Specs_1.2.pdf.
    
    ote there is also a complementary spec about PAM-CSM here:
    https://myesi.esi-group.com/ERF-HDF5/doc/ERF_CSM_RESULT_Specs_1.2.pdf.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkERFReader, obj, update, **traits)
    
    file_name = tvtk_base.vtk_file_name("", desc=\
        r"""
        Get/Set the name of the input file.
        """
    )

    def _file_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetFileName,
                        self.file_name)

    def _get_blocks_selection(self):
        return wrap_vtk(self._vtk_obj.GetBlocksSelection())
    blocks_selection = traits.Property(_get_blocks_selection, desc=\
        r"""
        
        """
    )

    def _get_stage(self):
        return self._vtk_obj.GetStage()
    stage = traits.Property(_get_stage, desc=\
        r"""
        Return the current stage.
        """
    )

    def _get_stages_selection(self):
        return wrap_vtk(self._vtk_obj.GetStagesSelection())
    stages_selection = traits.Property(_get_stages_selection, desc=\
        r"""
        Get the data array selections used to configure which data arrays
        are loaded by the reader.
        """
    )

    def _get_variables_selection(self):
        return wrap_vtk(self._vtk_obj.GetVariablesSelection())
    variables_selection = traits.Property(_get_variables_selection, desc=\
        r"""
        
        """
    )

    def enable_all_blocks(self):
        """
        enable_all_blocks(self) -> None
        C++: void enable_all_blocks()"""
        ret = self._vtk_obj.EnableAllBlocks()
        return ret
        

    def enable_all_variables(self):
        """
        enable_all_variables(self) -> None
        C++: void enable_all_variables()
        Enable all arrays to be read. Call forwarded to
        DataArraySelection.
        """
        ret = self._vtk_obj.EnableAllVariables()
        return ret
        

    def set_blocks_status(self, *args):
        """
        set_blocks_status(self, name:str, status:int) -> None
        C++: void set_blocks_status(const char *name, int status)"""
        ret = self._wrap_call(self._vtk_obj.SetBlocksStatus, *args)
        return ret

    def set_stages_status(self, *args):
        """
        set_stages_status(self, name:str, status:int) -> None
        C++: void set_stages_status(const char *name, int status)"""
        ret = self._wrap_call(self._vtk_obj.SetStagesStatus, *args)
        return ret

    def set_variables_status(self, *args):
        """
        set_variables_status(self, name:str, status:int) -> None
        C++: void set_variables_status(const char *name, int status)
        Get/Set whether the point or cell array with the given name is
        enabled. Call forwarded to DataArraySelection.
        """
        ret = self._wrap_call(self._vtk_obj.SetVariablesStatus, *args)
        return ret

    _updateable_traits_ = \
    (('abort_execute', 'GetAbortExecute'), ('release_data_flag',
    'GetReleaseDataFlag'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'), ('file_name',
    'GetFileName'), ('abort_output', 'GetAbortOutput'), ('progress_text',
    'GetProgressText'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['abort_execute', 'debug', 'global_warning_display',
    'release_data_flag', 'abort_output', 'file_name', 'object_name',
    'progress_text'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(ERFReader, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit ERFReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['abort_output', 'file_name', 'object_name']),
            title='Edit ERFReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit ERFReader properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

