# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.partitioning_strategy import PartitioningStrategy


class NativePartitioningStrategy(PartitioningStrategy):
    r"""
    NativePartitioningStrategy - A partitioning strategy based on load
    balancing geometric bounding boxes as cuts of a data set
    
    Superclass: PartitioningStrategy
    
    This strategy is the traditional strategy used in the
    RedistributeDataSetFilter which has been refactored into this
    class. It is based on cuting up the data set into equally balanced
    (in terms of cell number) bounding boxes that then get distributed to
    all ranks.
    
    The load balancing attempts to balance the number of cells per target
    partition approximately. It uses a DIY-based kdtree implementation
    that balances the cell centers among requested number of partitions.
    Current implementation only supports power-of-2 target partition. If
    a non-power of two value is specified for `number_of_partitions`, then
    the load balancing simply uses the power-of-two greater than the
    requested value. The bounding boxes for the kdtree leaf nodes are
    then used to redistribute the data.
    
    Alternatively a collection of bounding boxes may be provided that can
    be used to distribute the data instead of computing them (see
    `use_explicit_cuts` and `set_explicit_cuts`). When explicit cuts are
    specified, it is possible use those cuts strictly or to expand boxes
    on the edge to fit the domain of the input dataset. This can be
    controlled by `expand_explicit_cuts_for_input_domain`.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkNativePartitioningStrategy, obj, update, **traits)
    
    expand_explicit_cuts = tvtk_base.true_bool_trait(desc=\
        r"""
        When using explicit cuts, it possible that the bounding box
        defined by all the cuts is smaller than the input's bounds. In
        that case, the filter can automatically expand the edge boxes to
        include the input bounds to avoid clipping of the input dataset
        on the external faces of the combined bounding box.
        
        Default is true, that is explicit cuts will automatically be
        expanded.
        """
    )

    def _expand_explicit_cuts_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetExpandExplicitCuts,
                        self.expand_explicit_cuts_)

    load_balance_across_all_blocks = tvtk_base.true_bool_trait(desc=\
        r"""
        When use_explicit_cuts is false, and input is a
        `vtkpartitioned_data_set_collection`, set this to true to generate
        cuts for load balancing using all the datasets in the
        PartitionedDataSetCollection.
        
        Default is true.
        """
    )

    def _load_balance_across_all_blocks_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetLoadBalanceAcrossAllBlocks,
                        self.load_balance_across_all_blocks_)

    use_explicit_cuts = tvtk_base.false_bool_trait(desc=\
        r"""
        Specify whether to compute the load balancing automatically or
        use explicitly provided cuts. Set to false (default) to
        automatically compute the cuts to use for redistributing the
        dataset.
        """
    )

    def _use_explicit_cuts_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetUseExplicitCuts,
                        self.use_explicit_cuts_)

    def get_explicit_cut(self, *args):
        """
        get_explicit_cut(self, index:int) -> BoundingBox
        C++: const BoundingBox &get_explicit_cut(int index)"""
        ret = self._wrap_call(self._vtk_obj.GetExplicitCut, *args)
        return wrap_vtk(ret)

    def _get_number_of_explicit_cuts(self):
        return self._vtk_obj.GetNumberOfExplicitCuts()
    number_of_explicit_cuts = traits.Property(_get_number_of_explicit_cuts, desc=\
        r"""
        
        """
    )

    def add_explicit_cut(self, *args):
        """
        add_explicit_cut(self, bbox:BoundingBox) -> None
        C++: void add_explicit_cut(const BoundingBox &bbox)
        add_explicit_cut(self, bbox:(float, float, float, float, float,
            float)) -> None
        C++: void add_explicit_cut(const double bbox[6])"""
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.AddExplicitCut, *my_args)
        return ret

    def remove_all_explicit_cuts(self):
        """
        remove_all_explicit_cuts(self) -> None
        C++: void remove_all_explicit_cuts()"""
        ret = self._vtk_obj.RemoveAllExplicitCuts()
        return ret
        

    _updateable_traits_ = \
    (('expand_explicit_cuts', 'GetExpandExplicitCuts'),
    ('load_balance_across_all_blocks', 'GetLoadBalanceAcrossAllBlocks'),
    ('use_explicit_cuts', 'GetUseExplicitCuts'), ('debug', 'GetDebug'),
    ('global_warning_display', 'GetGlobalWarningDisplay'),
    ('number_of_partitions', 'GetNumberOfPartitions'), ('object_name',
    'GetObjectName'), ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'expand_explicit_cuts', 'global_warning_display',
    'load_balance_across_all_blocks', 'use_explicit_cuts',
    'number_of_partitions', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(NativePartitioningStrategy, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit NativePartitioningStrategy properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View((['expand_explicit_cuts', 'load_balance_across_all_blocks',
            'use_explicit_cuts'], [], ['number_of_partitions', 'object_name']),
            title='Edit NativePartitioningStrategy properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit NativePartitioningStrategy properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

