# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.object import Object


class ObjectManager(Object):
    r"""
    ObjectManager - ObjectManager maintains internal instances of
    Serializer and a Deserializer to serialize and deserialize VTK
    objects respectively.
    
    Superclass: Object
    
    The ObjectManager facilitates:
    1. serialization of objects by registering them, updating their
       state, and providing methods to retrieve both the serialized data
       (blobs) and object states based on their unique identifiers.
    2. deserialization of objects by registering their states and data
       (blobs) and constructing or updating VTK objects based on MTime.
    
    @sa ObjectManager
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkObjectManager, obj, update, **traits)
    
    def get_all_dependencies(self, *args):
        """
        get_all_dependencies(self, identifier:int) -> (int, ...)
        C++: std::vector<TypeUInt32> get_all_dependencies(
            TypeUInt32 identifier)
        Returns a non-empty vector of identifiers of all objects that
        depend on an object with the given identifier. Returns an empty
        vector if there are no dependents. When the root string is empty,
        the entire dependency tree is returned as a flat vector of
        identifiers.
        """
        ret = self._wrap_call(self._vtk_obj.GetAllDependencies, *args)
        return ret

    def get_blob(self, *args):
        """
        get_blob(self, hash:str) -> TypeUInt8Array
        C++: SmartPointer<vtkTypeUInt8Array> get_blob(
            const std::string &hash)
        Returns a blob stored at `hash`.
        """
        ret = self._wrap_call(self._vtk_obj.GetBlob, *args)
        return wrap_vtk(ret)

    def get_blob_hashes(self, *args):
        """
        get_blob_hashes(self, ids:(int, ...)) -> (str, ...)
        C++: std::vector<std::string> get_blob_hashes(
            const std::vector<TypeUInt32> &ids)
        Returns a non-empty vector of hash strings that correspond to
        blobs used by the registered objects at each identifier in `ids`.
        """
        ret = self._wrap_call(self._vtk_obj.GetBlobHashes, *args)
        return ret

    def _get_deserializer(self):
        return wrap_vtk(self._vtk_obj.GetDeserializer())
    deserializer = traits.Property(_get_deserializer, desc=\
        r"""
        
        """
    )

    def get_id(self, *args):
        """
        get_id(self, objectBase:ObjectBase) -> int
        C++: TypeUInt32 get_id(
            SmartPointer<vtkObjectBase> objectBase)
        Get the identifier for `object`. Returns an integer >=0 if
        `object` was previously registered directly or indirectly i.e, as
        a dependency of another registered object.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.GetId, *my_args)
        return ret

    def get_object_at_id(self, *args):
        """
        get_object_at_id(self, id:int) -> ObjectBase
        C++: SmartPointer<vtkObjectBase> get_object_at_id(
            TypeUInt32 id)
        Get object at `id`. Returns `nullptr` if there is no object
        registered directly or indirectly at `id`.
        """
        ret = self._wrap_call(self._vtk_obj.GetObjectAtId, *args)
        return wrap_vtk(ret)

    def _get_serializer(self):
        return wrap_vtk(self._vtk_obj.GetSerializer())
    serializer = traits.Property(_get_serializer, desc=\
        r"""
        
        """
    )

    def get_state(self, *args):
        """
        get_state(self, id:int) -> str
        C++: std::string get_state(TypeUInt32 id)
        Get state of the object at `id`. Returns a non empty json valid
        string if an object registered directly or indirectly at `id` has
        a state.
        """
        ret = self._wrap_call(self._vtk_obj.GetState, *args)
        return ret

    def _get_total_blob_memory_usage(self):
        return self._vtk_obj.GetTotalBlobMemoryUsage()
    total_blob_memory_usage = traits.Property(_get_total_blob_memory_usage, desc=\
        r"""
        
        """
    )

    def _get_total_vtk_data_object_memory_usage(self):
        return self._vtk_obj.GetTotalVTKDataObjectMemoryUsage()
    total_vtk_data_object_memory_usage = traits.Property(_get_total_vtk_data_object_memory_usage, desc=\
        r"""
        
        """
    )

    def clear(self):
        """
        clear(self) -> None
        C++: void clear()
        Reset to initial state. All registered objects are removed and no
        longer tracked. All registered states are also removed. All
        registered blobs are also removed.
        """
        ret = self._vtk_obj.Clear()
        return ret
        

    def export(self, *args):
        """
        export(self, filename:str, indentLevel:int=-1, indentChar:str=...)
             -> None
        C++: void export(const std::string &filename, int indentLevel=-1,
            char indentChar=' ')
        Writes state of all registered objects to `filename.json` The
        blobs are written into `filename.blobs.json`.
        """
        ret = self._wrap_call(self._vtk_obj.Export, *args)
        return ret

    def import_(self, *args):
        """
        import(self, stateFileName:str, blobFileName:str) -> None
        C++: void import(const std::string &stateFileName,
            const std::string &blobFileName)
        Reads state from state file and blobs from blob file. This clears
        existing states, objects, blobs, imports data from the two files
        and updates objects from the states.
        """
        ret = self._wrap_call(self._vtk_obj.Import, *args)
        return ret

    def initialize(self):
        """
        initialize(self) -> bool
        C++: virtual bool initialize()
        Loads the default (de)serialization handlers and constructors for
        VTK classes
        """
        ret = self._vtk_obj.Initialize()
        return ret
        

    def initialize_default_handlers(self):
        """
        initialize_default_handlers(self) -> bool
        C++: bool initialize_default_handlers()"""
        ret = self._vtk_obj.InitializeDefaultHandlers()
        return ret
        

    def prune_unused_blobs(self):
        """
        prune_unused_blobs(self) -> None
        C++: void prune_unused_blobs()
        Removes all `blob`(s) whose `hash` is not found in the state of
        any object registered directly or indirectly.
        """
        ret = self._vtk_obj.PruneUnusedBlobs()
        return ret
        

    def ROOT(self):
        """
        root() -> int
        C++: static TypeUInt32 root()"""
        ret = self._vtk_obj.ROOT()
        return ret
        

    def register_blob(self, *args):
        """
        register_blob(self, hash:str, blob:TypeUInt8Array) -> bool
        C++: bool register_blob(const std::string &hash,
            SmartPointer<vtkTypeUInt8Array> blob)
        Specifies a `blob` for `hash`. Returns `true` if the `blob` is
        valid and successfully registered, `false` otherwise.
        """
        my_args = deref_array(args, [('string', 'vtkTypeUInt8Array')])
        ret = self._wrap_call(self._vtk_obj.RegisterBlob, *my_args)
        return ret

    def register_object(self, *args):
        """
        register_object(self, objectBase:ObjectBase) -> int
        C++: TypeUInt32 register_object(
            SmartPointer<vtkObjectBase> objectBase)
        Adds `object` into an internal container and returns a unique
        identifier. The identifier can be used in any of the methods that
        accept `id` or a vector of `id`.
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.RegisterObject, *my_args)
        return ret

    def register_state(self, *args):
        """
        register_state(self, state:str) -> bool
        C++: bool register_state(const std::string &state)
        Adds `state` into an internal container and returns a unique
        identifier. The state
        1. must be valid json.
        2. must have a key-value pair `{'Id': n}` where n is an integer
           of type `std::string`.
        """
        ret = self._wrap_call(self._vtk_obj.RegisterState, *args)
        return ret

    def un_register_blob(self, *args):
        """
        un_register_blob(self, hash:str) -> bool
        C++: bool un_register_blob(const std::string &hash)
        Removes a `blob` stored at `hash`.
        """
        ret = self._wrap_call(self._vtk_obj.UnRegisterBlob, *args)
        return ret

    def un_register_object(self, *args):
        """
        un_register_object(self, identifier:int) -> bool
        C++: bool un_register_object(TypeUInt32 identifier)
        Removes an object and it's state. Returns true if an object
        exists at `id` and it was removed, false otherwise.
        """
        ret = self._wrap_call(self._vtk_obj.UnRegisterObject, *args)
        return ret

    def un_register_state(self, *args):
        """
        un_register_state(self, identifier:int) -> bool
        C++: bool un_register_state(TypeUInt32 identifier)
        Removes a state at `id`.
        """
        ret = self._wrap_call(self._vtk_obj.UnRegisterState, *args)
        return ret

    def update_object_from_state(self, *args):
        """
        update_object_from_state(self, state:str) -> None
        C++: void update_object_from_state(const std::string &state)
        Deserialize the state into vtk object.
        """
        ret = self._wrap_call(self._vtk_obj.UpdateObjectFromState, *args)
        return ret

    def update_objects_from_states(self):
        """
        update_objects_from_states(self) -> None
        C++: void update_objects_from_states()
        Deserialize registered states into vtk objects.
        """
        ret = self._vtk_obj.UpdateObjectsFromStates()
        return ret
        

    def update_state_from_object(self, *args):
        """
        update_state_from_object(self, identifier:int) -> None
        C++: void update_state_from_object(TypeUInt32 identifier)
        Serialize object at `identifier` into the state.
        """
        ret = self._wrap_call(self._vtk_obj.UpdateStateFromObject, *args)
        return ret

    def update_states_from_objects(self):
        """
        update_states_from_objects(self) -> None
        C++: void update_states_from_objects()
        Serialize registered objects into vtk states.
        """
        ret = self._vtk_obj.UpdateStatesFromObjects()
        return ret
        

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(ObjectManager, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit ObjectManager properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit ObjectManager properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit ObjectManager properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

