# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.implicit_function import ImplicitFunction


class Plane(ImplicitFunction):
    r"""
    Plane - perform various plane computations
    
    Superclass: ImplicitFunction
    
    Plane provides methods for various plane computations. These
    include projecting points onto a plane, evaluating the plane
    equation, and returning plane normal. Plane is a concrete
    implementation of the abstract class ImplicitFunction.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkPlane, obj, update, **traits)
    
    normal = traits.Array(enter_set=True, auto_set=False, shape=(None,), dtype="float", value=(0.0, 0.0, 1.0), cols=3, desc=\
        r"""
        Set/get plane normal. Plane is defined by point and normal.
        """
    )

    def _normal_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetNormal,
                        self.normal)

    origin = traits.Array(enter_set=True, auto_set=False, shape=(None,), dtype="float", value=(0.0, 0.0, 0.0), cols=3, desc=\
        r"""
        Set/get point through which plane passes. Plane is defined by
        point and normal.
        """
    )

    def _origin_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetOrigin,
                        self.origin)

    def compute_best_fitting_plane(self, *args):
        """
        compute_best_fitting_plane(pts:Points, origin:[float, ...],
            normal:[float, ...]) -> bool
        C++: static bool compute_best_fitting_plane(Points *pts,
            double *origin, double *normal)
        Given a set of points calculate the best-fitting origin and
        normal for the plane. The origin will be the centroid of the
        points. The normal is determined by using the covariance matrix
        of the points relative to the centroid. Returns true if
        successful. If not successful the origin will still contain the
        centroid and the normal will point into z-direction.
        """
        my_args = deref_array(args, [('vtkPoints', 'tuple', 'tuple')])
        ret = self._wrap_call(self._vtk_obj.ComputeBestFittingPlane, *my_args)
        return ret

    def distance_to_plane(self, *args):
        """
        distance_to_plane(x:[float, float, float], n:[float, float, float],
            p0:[float, float, float]) -> float
        C++: static double distance_to_plane(double x[3], double n[3],
            double p0[3])
        distance_to_plane(self, x:[float, float, float]) -> float
        C++: double distance_to_plane(double x[3])
        Return the distance of a point x to a plane defined by n(x-p0) =
        0. The normal n[3] must be magnitude=1.
        """
        ret = self._wrap_call(self._vtk_obj.DistanceToPlane, *args)
        return ret

    def evaluate(self, *args):
        """
        evaluate(normal:[float, float, float], origin:[float, float,
            float], x:[float, float, float]) -> float
        C++: static double evaluate(double normal[3], double origin[3],
            double x[3])
        Quick evaluation of plane equation n(x-origin)=0.
        """
        ret = self._wrap_call(self._vtk_obj.Evaluate, *args)
        return ret

    def generalized_project_point(self, *args):
        """
        generalized_project_point(x:(float, float, float), origin:(float,
            float, float), normal:(float, float, float), xproj:[float,
            float, float]) -> None
        C++: static void generalized_project_point(const double x[3],
            const double origin[3], const double normal[3],
            double xproj[3])
        generalized_project_point(self, x:(float, float, float),
            xproj:[float, float, float]) -> None
        C++: void generalized_project_point(const double x[3],
            double xproj[3])
        Project a point x onto plane defined by origin and normal. The
        projected point is returned in xproj. NOTE : normal does NOT have
        to have magnitude 1.
        """
        ret = self._wrap_call(self._vtk_obj.GeneralizedProjectPoint, *args)
        return ret

    def intersect_with_finite_plane(self, *args):
        """
        intersect_with_finite_plane(n:[float, float, float], o:[float, float,
             float], pOrigin:[float, float, float], px:[float, float,
            float], py:[float, float, float], x0:[float, float, float],
            x1:[float, float, float]) -> int
        C++: static int intersect_with_finite_plane(double n[3], double o[3],
             double pOrigin[3], double px[3], double py[3], double x0[3],
            double x1[3])
        intersect_with_finite_plane(self, pOrigin:[float, float, float],
            px:[float, float, float], py:[float, float, float], x0:[float,
             float, float], x1:[float, float, float]) -> int
        C++: int intersect_with_finite_plane(double pOrigin[3], double px[3],
             double py[3], double x0[3], double x1[3])
        Given two planes, one infinite and one finite, defined by the
        normal n and point o (infinite plane), and the second finite
        plane1 defined by the three points (pOrigin,px,py), compute a
        line of intersection (if any). The line of intersection is
        defined by the return values (x0,x1). If there is no
        intersection, then zero is returned; otherwise non-zero. There
        are two variants of this method. The static function operates on
        the supplied function parameters; the non-static operates on this
        instance of Plane (and its associated origin and normal).
        """
        ret = self._wrap_call(self._vtk_obj.IntersectWithFinitePlane, *args)
        return ret

    def intersect_with_line(self, *args):
        """
        intersect_with_line(p1:(float, float, float), p2:(float, float,
            float), n:[float, float, float], p0:[float, float, float],
            t:float, x:[float, float, float]) -> int
        C++: static int intersect_with_line(const double p1[3],
            const double p2[3], double n[3], double p0[3], double &t,
            double x[3])
        intersect_with_line(self, p1:(float, float, float), p2:(float,
            float, float), t:float, x:[float, float, float]) -> int
        C++: int intersect_with_line(const double p1[3], const double p2[3],
             double &t, double x[3])
        Given a line defined by the two points p1,p2; and a plane defined
        by the normal n and point p0, compute an intersection. The
        parametric coordinate along the line is returned in t, and the
        coordinates of intersection are returned in x. A zero is returned
        if the plane and line do not intersect between (0<=t<=1). If the
        plane and line are parallel, zero is returned and t is set to
        VTK_LARGE_DOUBLE.
        """
        ret = self._wrap_call(self._vtk_obj.IntersectWithLine, *args)
        return ret

    def project_point(self, *args):
        """
        project_point(x:(float, float, float), origin:(float, float, float)
            , normal:(float, float, float), xproj:[float, float, float])
            -> None
        C++: static void project_point(const double x[3],
            const double origin[3], const double normal[3],
            double xproj[3])
        project_point(self, x:(float, float, float), xproj:[float, float,
            float]) -> None
        C++: void project_point(const double x[3], double xproj[3])
        Project a point x onto plane defined by origin and normal. The
        projected point is returned in xproj. NOTE : normal assumed to
        have magnitude 1.
        """
        ret = self._wrap_call(self._vtk_obj.ProjectPoint, *args)
        return ret

    def project_vector(self, *args):
        """
        project_vector(v:(float, float, float), origin:(float, float,
            float), normal:(float, float, float), vproj:[float, float,
            float]) -> None
        C++: static void project_vector(const double v[3],
            const double origin[3], const double normal[3],
            double vproj[3])
        project_vector(self, v:(float, float, float), vproj:[float, float,
            float]) -> None
        C++: void project_vector(const double v[3], double vproj[3])
        Project a vector v onto plane defined by origin and normal. The
        projected vector is returned in vproj.
        """
        ret = self._wrap_call(self._vtk_obj.ProjectVector, *args)
        return ret

    def push(self, *args):
        """
        push(self, distance:float) -> None
        C++: void push(double distance)
        Translate the plane in the direction of the normal by the
        distance specified.  Negative values move the plane in the
        opposite direction.
        """
        ret = self._wrap_call(self._vtk_obj.Push, *args)
        return ret

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('normal', 'GetNormal'), ('origin',
    'GetOrigin'), ('object_name', 'GetObjectName'), ('reference_count',
    'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'normal', 'object_name',
    'origin'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(Plane, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit Plane properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['normal', 'object_name', 'origin']),
            title='Edit Plane properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit Plane properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

