# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk

nan = float('nan')


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

inf = float('inf')

from tvtk.tvtk_classes.object import Object


class ResourceStream(Object):
    r"""
    ResourceStream - Abstract class used for custom streams
    
    Superclass: Object
    
    ResourceStream provides a high-level interface for manipulating
    custom streams. This class is designed to be used instead of
    "set_file_name" and "set_input_string" or similar that exists in most of
    reader or reader-like classes in VTK.
    
    ResourceStream only support unformatted input, user must use
    external solution to format the input, such as sscanf or scnlib,
    fast_float or strtoX
    
    ResourceStream **may** be support seeking, code that uses
    ResourceStream should take in account this, and support for both
    seekable stream and not seekable streams, if possible.
    `stream.support_seek()` can be used for support checking.
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkResourceStream, obj, update, **traits)
    
    def end_of_stream(self):
        """
        end_of_stream(self) -> bool
        C++: virtual bool end_of_stream()
        Check if the stream reached an end.
        
        The stream may be out of range (EOF) if a Seek call move before
        stream begin (0), or after stream end (max) Seek on an
        end_of_stream stream may restore it back to a valid state. Read on
        an invalid stream must do nothing and return 0.
        
        @return Return true if the stream reached the end of input, false
        otherwise.
        """
        ret = self._vtk_obj.EndOfStream()
        return ret
        

    def read(self, *args):
        """
        read(self, buffer:Pointer, bytes:int) -> int
        C++: virtual std::size_t read(void *buffer, std::size_t bytes)
        Read data form the stream
        
        Read at most `bytes` bytes from input stream. If this function
        reads less that `bytes` bytes, then end_of_stream must return true.
        
        @param buffer User provided storage buffer, may be nullptr if
            bytes == 0
        @param bytes Maximum amount of bytes to read.
        @return The number of bytes read, may be 0.
        """
        ret = self._wrap_call(self._vtk_obj.Read, *args)
        return ret

    def seek(self, *args):
        """
        seek(self, pos:int, dir:SeekDirection) -> int
        C++: virtual TypeInt64 seek(TypeInt64 pos,
            SeekDirection dir)
        Move the stream cursor position
        
        Base version does nothing and returns 0. This function does not
        check if the position is out of range. end_of_file() result must
        only change after a call of Read with non-zero size.
        
        @return Current position in stream.
        """
        ret = self._wrap_call(self._vtk_obj.Seek, *args)
        return ret

    def support_seek(self):
        """
        support_seek(self) -> bool
        C++: bool support_seek()
        Check if stream supports Seek and Tell functions
        
        @return true if Seek and Tell functions are supported.
        """
        ret = self._vtk_obj.SupportSeek()
        return ret
        

    def tell(self):
        """
        tell(self) -> int
        C++: virtual TypeInt64 tell()
        Get the stream cursor position
        
        Base version calls `Seek(0, std::ios_base::cur)`. If seeking is
        unsupported, return 0. Subclasses may reimplement it to provide a
        more efficient version.
        
        @return Current position in stream.
        """
        ret = self._vtk_obj.Tell()
        return ret
        

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_warning_display',
    'GetGlobalWarningDisplay'), ('object_name', 'GetObjectName'),
    ('reference_count', 'GetReferenceCount'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'object_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(ResourceStream, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit ResourceStream properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['object_name']),
            title='Edit ResourceStream properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit ResourceStream properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

